@@ -1,7 +1,20 @@
+from __future__ import absolute_import
+
 from docstructure import SITE_STRUCTURE, HREF_MAP, BASENAME_MAP
-from lxml.etree import (parse, fromstring, tostring, ElementTree,
+from lxml.etree import (parse, fromstring, ElementTree,
                         Element, SubElement, XPath, XML)
-import os, shutil, re, sys, copy, time
+import os
+import re
+import sys
+import copy
+import shutil
+import textwrap
+import subprocess
+
+try:
+    from io import open as open_file
+except ImportError:
+    from codecs import open as open_file
 
 RST2HTML_OPTIONS = " ".join([
     '--no-toc-backlinks',
@@ -11,8 +24,10 @@ RST2HTML_OPTIONS = " ".join([
     ])
 
 XHTML_NS = 'http://www.w3.org/1999/xhtml'
-htmlnsmap = {"h" : XHTML_NS}
+htmlnsmap = {"h": XHTML_NS}
 
+find_head = XPath("/h:html/h:head[1]", namespaces=htmlnsmap)
+find_body = XPath("/h:html/h:body[1]", namespaces=htmlnsmap)
 find_title = XPath("/h:html/h:head/h:title/text()", namespaces=htmlnsmap)
 find_title_tag = XPath("/h:html/h:head/h:title", namespaces=htmlnsmap)
 find_headings = XPath("//h:h1[not(@class)]//text()", namespaces=htmlnsmap)
@@ -23,6 +38,7 @@ find_page_end = XPath("/h:html/h:body/h:div[last()]", namespaces=htmlnsmap)
 find_words = re.compile('(\w+)').findall
 replace_invalid = re.compile(r'[-_/.\s\\]').sub
 
+
 def make_menu_section_head(section, menuroot):
     section_id = section + '-section'
     section_head = menuroot.xpath("//ul[@id=$section]/li", section=section_id)
@@ -35,6 +51,7 @@ def make_menu_section_head(section, menuroot):
         section_head = section_head[0]
     return section_head
 
+
 def build_menu(tree, basename, section_head):
     page_title = find_title(tree)
     if page_title:
@@ -44,6 +61,7 @@ def build_menu(tree, basename, section_head):
     build_menu_entry(page_title, basename+".html", section_head,
                      headings=find_headings(tree))
 
+
 def build_menu_entry(page_title, url, section_head, headings=None):
     page_id = replace_invalid(' ', os.path.splitext(url)[0]) + '-menu'
     ul = SubElement(section_head, "ul", {"class":"menu foreign", "id":page_id})
@@ -62,12 +80,13 @@ def build_menu_entry(page_title, url, section_head, headings=None):
                 ref = None
             if ref is None:
                 ref = '-'.join(find_words(replace_invalid(' ', heading.lower())))
-            a  = SubElement(li, "a", href=url+'#'+ref)
+            a = SubElement(li, "a", href=url+'#'+ref)
             a.text = heading
 
+
 def merge_menu(tree, menu, name):
     menu_root = copy.deepcopy(menu)
-    tree.getroot()[1][0].insert(0, menu_root) # html->body->div[class=document]
+    tree.getroot()[1][0].insert(0, menu_root)  # html->body->div[class=document]
     for el in menu_root.iter():
         tag = el.tag
         if tag[0] != '{':
@@ -83,6 +102,7 @@ def merge_menu(tree, menu, name):
                         replace("foreign", "current"))
     return tree
 
+
 def inject_flatter_button(tree):
     head = tree.xpath('h:head[1]', namespaces=htmlnsmap)[0]
     script = SubElement(head, '{%s}script' % XHTML_NS, type='text/javascript')
@@ -106,11 +126,57 @@ def inject_flatter_button(tree):
         '</p>'
         ))
 
+
+def inject_donate_buttons(lxml_path, rst2html_script, tree):
+    command = ([sys.executable, rst2html_script]
+               + RST2HTML_OPTIONS.split() + [os.path.join(lxml_path, 'README.rst')])
+    rst2html = subprocess.Popen(command, stdout=subprocess.PIPE)
+    stdout, _ = rst2html.communicate()
+    readme = fromstring(stdout)
+
+    intro_div = tree.xpath('h:body//h:div[@id = "introduction"][1]',
+                           namespaces=htmlnsmap)[0]
+    support_div = readme.xpath('h:body//h:div[@id = "support-the-project"][1]',
+                               namespaces=htmlnsmap)[0]
+    intro_div.append(support_div)
+
+    legal = readme.xpath('h:body//h:div[@id = "legal-notice-for-donations"][1]',
+                         namespaces=htmlnsmap)[0]
+    last_div = tree.xpath('h:body//h:div//h:div', namespaces=htmlnsmap)[-1]
+    last_div.addnext(legal)
+
+
 def rest2html(script, source_path, dest_path, stylesheet_url):
     command = ('%s %s %s --stylesheet=%s --link-stylesheet %s > %s' %
                (sys.executable, script, RST2HTML_OPTIONS,
                 stylesheet_url, source_path, dest_path))
-    os.system(command)
+    subprocess.call(command, shell=True)
+
+
+def convert_changelog(lxml_path, changelog_file_path, rst2html_script, stylesheet_url):
+    f = open_file(os.path.join(lxml_path, 'CHANGES.txt'), 'r', encoding='utf-8')
+    try:
+        content = f.read()
+    finally:
+        f.close()
+
+    links = dict(LP='`%s <https://bugs.launchpad.net/lxml/+bug/%s>`_',
+                 GH='`%s <https://github.com/lxml/lxml/issues/%s>`_')
+    replace_tracker_links = re.compile('((LP|GH)#([0-9]+))').sub
+    def insert_link(match):
+        text, ref_type, ref_id = match.groups()
+        return links[ref_type] % (text, ref_id)
+    content = replace_tracker_links(insert_link, content)
+
+    command = [sys.executable, rst2html_script] + RST2HTML_OPTIONS.split() + [
+        '--link-stylesheet', '--stylesheet', stylesheet_url ]
+    out_file = open(changelog_file_path, 'wb')
+    try:
+        rst2html = subprocess.Popen(command, stdin=subprocess.PIPE, stdout=out_file)
+        rst2html.communicate(content.encode('utf8'))
+    finally:
+        out_file.close()
+
 
 def publish(dirname, lxml_path, release):
     if not os.path.exists(dirname):
@@ -127,11 +193,31 @@ def publish(dirname, lxml_path, release):
     changelog_basename = 'changes-%s' % release
     href_map['Release Changelog'] = changelog_basename + '.html'
 
+    menu_js = textwrap.dedent('''
+    function trigger_menu(event) {
+        var sidemenu = document.getElementById("sidemenu");
+        var classes = sidemenu.getAttribute("class");
+        classes = (classes.indexOf(" visible") === -1) ? classes + " visible" : classes.replace(" visible", "");
+        sidemenu.setAttribute("class", classes);
+        event.preventDefault();
+        event.stopPropagation();
+    }
+    function hide_menu() {
+        var sidemenu = document.getElementById("sidemenu");
+        var classes = sidemenu.getAttribute("class");
+        if (classes.indexOf(" visible") !== -1) {
+            sidemenu.setAttribute("class", classes.replace(" visible", ""));
+        }
+    }
+    ''')
+
     trees = {}
-    menu = Element("div", {"class":"sidemenu"})
+    menu = Element("div", {'class': 'sidemenu', 'id': 'sidemenu'})
+    SubElement(menu, 'div', {'class': 'menutrigger', 'onclick': 'trigger_menu(event)'}).text = "Menu"
+    menu_div = SubElement(menu, 'div', {'class': 'menu'})
     # build HTML pages and parse them back
     for section, text_files in SITE_STRUCTURE:
-        section_head = make_menu_section_head(section, menu)
+        section_head = make_menu_section_head(section, menu_div)
         for filename in text_files:
             if filename.startswith('@'):
                 # special menu entry
@@ -146,20 +232,22 @@ def publish(dirname, lxml_path, release):
                 outpath = os.path.join(dirname, outname)
 
                 rest2html(script, path, outpath, stylesheet_url)
-
                 tree = parse(outpath)
-                trees[filename] = (tree, basename, outpath)
 
+                if filename == 'main.txt':
+                    # inject donation buttons
+                    #inject_flatter_button(tree)
+                    inject_donate_buttons(lxml_path, script, tree)
+
+                trees[filename] = (tree, basename, outpath)
                 build_menu(tree, basename, section_head)
 
     # also convert CHANGES.txt
-    rest2html(script,
-              os.path.join(lxml_path, 'CHANGES.txt'),
-              os.path.join(dirname, 'changes-%s.html' % release),
-              '')
+    convert_changelog(lxml_path, os.path.join(dirname, 'changes-%s.html' % release),
+                      script, stylesheet_url)
 
     # generate sitemap from menu
-    sitemap = XML('''\
+    sitemap = XML(textwrap.dedent('''\
     <!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
     <html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en">
       <head>
@@ -174,20 +262,22 @@ def publish(dirname, lxml_path, release):
         <h1>Sitemap of lxml.de - Processing XML and HTML with Python</h1>
       </body>
     </html>
-    '''.replace('    ', ' '))
+    '''))
     sitemap_menu = copy.deepcopy(menu)
     SubElement(SubElement(sitemap_menu[-1], 'li'), 'a', href='http://lxml.de/files/').text = 'Download files'
-    sitemap[-1].append(sitemap_menu) # append to body
+    sitemap[-1].append(sitemap_menu)  # append to body
     ElementTree(sitemap).write(os.path.join(dirname, 'sitemap.html'))
 
     # integrate sitemap into the menu
-    SubElement(SubElement(menu[-1], 'li'), 'a', href='http://lxml.de/sitemap.html').text = 'Sitemap'
-
-    # inject flattr button
-    inject_flatter_button(trees['main.txt'][0])
+    SubElement(SubElement(menu_div[-1], 'li'), 'a', href='http://lxml.de/sitemap.html').text = 'Sitemap'
 
     # integrate menu into web pages
     for tree, basename, outpath in trees.itervalues():
+        head = find_head(tree)[0]
+        SubElement(head, 'script', type='text/javascript').text = menu_js
+        SubElement(head, 'meta', name='viewport', content="width=device-width, initial-scale=1")
+        find_body(tree)[0].set('onclick', 'hide_menu()')
+
         new_tree = merge_menu(tree, menu, basename)
         title = find_title_tag(new_tree)
         if title and title[0].text == 'lxml':
@@ -197,5 +287,6 @@ def publish(dirname, lxml_path, release):
                 heading[0].text = "lxml - XML and HTML with Python"
         new_tree.write(outpath)
 
+
 if __name__ == '__main__':
     publish(sys.argv[1], sys.argv[2], sys.argv[3])
